<?php
/**
 * 			Plugin Editor Switcher
 * @version			1.7.0
 * @package			Editor Switcher
 * @copyright		Copyright (C) 2009 Joomler!.net. All rights reserved.
 * @license			http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 *
 * @author			Yoshiki Kozaki(www.joomler.net)
 * @link			http://www.joomler.net/
 *
 */

// no direct access
defined('_JEXEC') or die;

jimport('joomla.plugin.plugin');

class plgEditorSwitcher extends JPlugin
{
	protected $_switchereditor = null;

	/**
	 * Constructor
	 *
	 * @access      protected
	 * @param       object  $subject The object to observe
	 * @param       array   $config  An array that holds the plugin configuration
	 * @since       1.5
	 */
	public function __construct(& $subject, $config)
	{
		$mainframe = & JFactory::getApplication();
		$switcher = JPluginHelper::getPlugin('editors', 'switcher');

		$editor = strtolower( $mainframe->getUserStateFromRequest('editorswitcher.editor', 'switcheditor', $switcher->params->get('default_editor', 'tinymce'), 'cmd') );
		if($editor == 'switcher'){
			$editor = $switcher->params->get('default_editor', 'none');
		}

		if(file_exists(dirname(dirname(__FILE__)).DS.$editor.DS.$editor. '.php') && JPluginHelper::isEnabled('editors', $editor)){
			$plugin = JPluginHelper::getPlugin('editors', $editor);
			$this->_setSwitcherEditor($subject, $config, $plugin);
		} else {
			$plugins = JPluginHelper::getPlugin('editors');
			$plugin = null;
			if(count($plugins)){
				foreach($plugins as $v){
					if($v->name != 'switcher'){
						$plugin = $v;
						break;
					}
				}
			}

			if($plugin){
				$this->_setSwitcherEditor($subject, $config, $plugin);
			} else {
				JError::raiseWarning('SOME_ERROR_CODE', JText::_('PLG_EDITOR_SWITCHER_EDITORWASNOTFOUND'));
			}
		}

		parent::__construct($subject, $config);
	}

	protected function _setSwitcherEditor(& $subject, & $config, $plugin)
	{
		$editor = $plugin->name;

		$config['params'] = $plugin->params;
		$config['name'] = strtolower($editor);
		$config['type'] = 'editor';
		require_once dirname(dirname(__FILE__)).DS.$editor.DS.$editor. '.php';
		$classname = 'plgEditor'. ucfirst($editor);
		$this->_switchereditor = new $classname($subject, $config);
	}

	protected function _setEditorSelector($current)
	{
		static $selector = null;
		
		if(is_null($selector)){
			$output = true;
			$db		= JFactory::getDbo();
			$authGroups = JFactory::getUser()->getAuthorisedGroups();

			$query = $db->getQuery(true);
			$query->select('element AS value, element AS text, access');
			$query->from('#__extensions');
			$query->where('folder = "editors" AND type = "plugin" AND enabled = 1 AND element <> "switcher"');
			$query->order('ordering, name');

			$db->setQuery( $query );
			$temp = $db->loadObjectList();
			if(count($temp) < 1){
				$selector = '';
				return $selector;
			}
			
			$editors = array();
			
			for($i = 0, $total = count($temp); $i < $total; $i++){
				$editor = & $temp[$i];
				if(!in_array($editor->access, $authGroups)) continue;
				$editor->text = str_replace('Editor - ', '', $editor->text);
				$editors[] = $editor;
			}
			
			if(count($editors) < 2){
				$selector = '';
				return $selector;
			}

			$uri = JURI::current();

			$action = $uri . (strpos('?', $uri)? '&':'?'). $this->_buildAction();
			$task = JRequest::getCmd('task');
			$section = JRequest::getCmd('section');

			JHtml::_('behavior.mootools');

			$js = 'onchange="if($(\'section\')) new Element(\'input\').setProperties({\'type\':\'hidden\',\'name\':\'section\',\'value\':\''. $section.'\'}).inject(this.form);this.form.task.value=\''. $task. '\';this.form.action=\''. $action. '\';this.form.submit();"';

			$selector = '<div id="switcherSelector" class="button2-left"><div class="blank" style="padding:0 3px 3px 7px;">'
			. JHtml::_('select.genericlist',   $editors, 'switcheditor'. '', 'class="modal-button" style="margin-top:3px;" '. $js, 'value', 'text', $current, 'jswitcheditor' ). '</div></div>';

			$document = & JFactory::getDocument();
			$js = "window.addEvent('domready', function(){if(document.id('editor-xtd-buttons')) document.id('switcherSelector').inject(document.id('editor-xtd-buttons'));});";
			$document->addScriptDeclaration($js);
		}

		return $selector;
	}

	protected function _buildAction()
	{
		$action = array();
		$vars = array(
			'option',
			'view',
			'task',
			'cid[]',
			'id',
			'type',
			'scope',
			'tmpl',
			'layout'
		);
		
		$addactions = JPluginHelper::getPlugin('editors', 'switcher')->params->get('addactions');
		if(!empty($addactions)){
			$addactions = explode("\n", $addactions);
			foreach($addactions as $ac){
				$ac = trim($ac);
				if(empty($ac)) continue;
				$vars[] = $ac;
			}
		}

		foreach($vars as $var){
			$value = null;
			if(strpos($var, '[]')){
				$temp = JRequest::getVar(str_replace('[]', '', $var), array(), '', 'array');
				if(count($temp)){
					$value = $temp[0];
				}
			} else {
				$value = JRequest::getCmd($var);
			}

			if($value){
				$action[] = $var. '='. $value;
			}

		}

		return implode('&', $action);
	}

	public function onInit()
	{
		if(is_callable(array($this->_switchereditor, 'onInit'))){
			return $this->_switchereditor->onInit();
		}
	}

	public function onGetContent( $editor ) {
		if(is_callable(array($this->_switchereditor, 'onGetContent'))){
			return $this->_switchereditor->onGetContent($editor);
		}
	}

	public function onSetContent( $editor, $html ) {
		if(is_callable(array($this->_switchereditor, 'onSetContent'))){
			return $this->_switchereditor->onSetContent( $editor, $html );
		}
	}

	public function onSave( $editor ) {
		if(is_callable(array($this->_switchereditor, 'onSave'))){
			return $this->_switchereditor->onSave( $editor );
		}
	}

	public function onDisplay($name, $content, $width, $height, $col, $row, $buttons = true, $id = null, $asset = null, $author = null)
	{
		if(is_callable(array($this->_switchereditor, 'onDisplay'))){
			return $this->_switchereditor->onDisplay($name, $content, $width, $height, $col, $row, $buttons, $id, $asset, $author). $this->_setEditorSelector($this->_switchereditor->_name);
		}
	}

	public function onGetInsertMethod($name)
	{
		if(is_callable(array($this->_switchereditor, 'onGetInsertMethod'))){
			return $this->_switchereditor->onGetInsertMethod($name);
		}
	}
}
